//
//  AppController.m
//  UpTime
//
//  Created by Milo on 20/06/2007.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "AppController.h"
#include <sys/sysctl.h>

NSString *MBCurrentUpTime = @"MBCurrentUpTime";
NSString *MBDateOfLastRestart = @"MBDateOfLastRestart";
NSString *MBLongestRecordedUpTime = @"MBLongestRecordedUpTime";
NSString *MBLastTenRecordedUpTimes = @"MBLastTenRecordedUpTimes";


@implementation AppController

//  NSObject
- (id)init;
{
	if ([super init] == nil)
		return nil;
	
	NSDate *bootTime = [NSDate dateWithBootTime];
	if (bootTime != nil && [[NSUserDefaults standardUserDefaults] objectForKey:MBDateOfLastRestart] != nil && [[[NSUserDefaults standardUserDefaults] objectForKey:MBDateOfLastRestart] isEqualToDate:bootTime] == NO)
	{
		NSMutableArray *lastTenUpTimes = [[self lastTenRecordedUpTimes] mutableCopy];
		if (lastTenUpTimes == nil)
			lastTenUpTimes = [[NSMutableArray alloc] initWithCapacity:10];
		[lastTenUpTimes insertObject:[self currentUpTime] atIndex:0];
		if ([lastTenUpTimes count] > 10)
			[lastTenUpTimes removeLastObject];
		[self setLastTenRecordedUpTimes:lastTenUpTimes];
		[lastTenUpTimes release];
	}
	
	if (bootTime != nil)
		[[NSUserDefaults standardUserDefaults] setObject:bootTime forKey:MBDateOfLastRestart];
	
	[self updateCurrentUpTime];
	[NSTimer scheduledTimerWithTimeInterval:60.0 target:self selector:@selector(updateCurrentUpTime) userInfo:nil repeats:YES];
		
	return self;
}

#pragma mark -

- (void)updateCurrentUpTime;
{
	[self setCurrentUpTime:[NSNumber numberWithCurrentUpTime]];
}

- (NSNumber *)currentUpTime;
{
	return [[NSUserDefaults standardUserDefaults] objectForKey:MBCurrentUpTime];
}

- (void)setCurrentUpTime:(NSNumber *)newUpTime;
{
	[[NSUserDefaults standardUserDefaults] setObject:newUpTime forKey:MBCurrentUpTime];
	
	if ([self longestRecordedUpTime] == nil || [newUpTime compare:[self longestRecordedUpTime]] == NSOrderedDescending)
		[self setLongestRecordedUpTime:newUpTime];
}

- (NSNumber *)longestRecordedUpTime;
{
	return [[NSUserDefaults standardUserDefaults] objectForKey:MBLongestRecordedUpTime];
}

- (void)setLongestRecordedUpTime:(NSNumber *)newUpTime;
{
	[[NSUserDefaults standardUserDefaults] setObject:newUpTime forKey:MBLongestRecordedUpTime];
}

- (NSArray *)lastTenRecordedUpTimes;
{
	return [[NSUserDefaults standardUserDefaults] objectForKey:MBLastTenRecordedUpTimes];
}

- (void)setLastTenRecordedUpTimes:(NSArray *)newUpTimes;
{
	[[NSUserDefaults standardUserDefaults] setObject:newUpTimes forKey:MBLastTenRecordedUpTimes];
}

@end


#pragma mark -
@implementation NSNumber (MBUpTimeAdditions)

+ (NSNumber *)numberWithCurrentUpTime;
{
	return [self numberWithDouble:(double)UnsignedWideToUInt64(AbsoluteToNanoseconds(UpTime())) / 1000000000];	// Uptime in seconds
}

@end


#pragma mark -
@implementation NSDate (MBUpTimeAdditions)

+ (NSDate *)dateWithBootTime;
{
	struct timeval bootTime;
	size_t length = sizeof(bootTime);
	int name[2] = {CTL_KERN, KERN_BOOTTIME};
	if (sysctl(name, 2, &bootTime, &length, NULL, 0) != -1)
		return [NSDate dateWithTimeIntervalSince1970:(NSTimeInterval)bootTime.tv_sec];
		
	perror("sysctl");
	return nil;
}

@end